# -*- encoding: utf-8 -*-

# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

require File.expand_path('../spec_helper', __FILE__)

describe Plangrade::Client do

  before :each do
    @configuration = {
      :site_url      => 'https://plangrade.com',
      :client_id     => 'PRbTcg9qjgKsp4jjpm1pw',
      :client_secret => 'Xn7kp7Ly0TCY4GtZWkmSsqGEPg10DmMADyjWkf2U',
      :access_token  => 'TolNOFka9Uls2DxahNi78A',
      :connection_options => { :max_redirects => 5, :use_ssl => true },
      :default_headers    => {"Accept"=>"application/json", "User-Agent"=>"Plangrade Ruby Gem #{Plangrade::Ruby::VERSION}"}
    }

    Plangrade.configure do |config|
      @configuration.each do |key, val|
        config.send("#{key}=", val)
      end
    end
  end

  after :each do
    Plangrade.reset!
  end

  subject { Plangrade::Client.new(@configuration) }

  context "with module configuration" do

    before :each do
      @default_conf = {
        :site_url         => nil,
        :client_id        => nil,
        :client_secret    => nil,
        :access_token     => nil,
        :default_headers  => {"Accept"=>"application/json", "User-Agent"=>"Plangrade Ruby Gem #{Plangrade::Ruby::VERSION}"},
        :connection_options => {}
      }
      Plangrade.configure do |config|
        Plangrade::Configurable.keys.each do |key|
          config.send("#{key}=", @default_conf[key])
        end
      end
    end

    after do
      Plangrade.reset!
    end

    it "inherits the module configuration" do
      client = Plangrade::Client.new
      @default_conf.each do |key, value|
        expect(client.instance_variable_get(:"@#{key}")).to eq value
      end
    end

    context "with initialization options" do
      before(:all) do
        @conf = {
          :site_url           => 'https://plangrade.com',
          :client_id          => 'PRbTcg9qjgKsp4jjpm1pw',
          :client_secret      => 'Xn7kp7Ly0TCY4GtZWkmSsqGEPg10DmMADyjWkf2U',
          :access_token       => 'TolNOFka9Uls2DxahNi78A',
          :default_headers    => {"Accept"=>"application/json", "User-Agent"=>"Plangrade Ruby Gem #{Plangrade::Ruby::VERSION}"},
          :connection_options => { :max_redirects => 5, :use_ssl => true }
        }
      end

      context "during initialization" do
        it "overrides the module configuration" do
          client = Plangrade::Client.new(@conf)
          Plangrade::Configurable.keys.each do |key|
            expect(client.instance_variable_get(:"@#{key}")).to eq @conf[key]
          end
        end
      end

      context "after initialization" do
        it "overrides the module configuration" do
          Plangrade.configure {|c| c.connection_options = {}}
          client = Plangrade::Client.new
          client.configure do |config|
            @conf.each do |key, value|
              config.send("#{key}=", value)
            end
          end
          Plangrade::Configurable.keys.each do |key|
            expect(client.instance_variable_get(:"@#{key}")).to eq @conf[key]
          end
        end
      end
    end
  end



  describe "#connection_options" do
    context "with default connection options" do
      it "returns empty hash" do
        expect(subject.connection_options).to eq({ :max_redirects => 5, :use_ssl => true })
      end
    end

    context "with custom connection options" do
      it "returns default options" do
        subject.connection_options = { :max_redirects => 5, :use_ssl => true }
        expect(subject.connection_options).to eq({:max_redirects => 5, :use_ssl => true})
      end
    end
  end

  describe "#request" do
    context "when method is not supported" do
      it "raises an error" do
        expect {subject.send(:request, :patch, '/')}.to raise_error
      end
    end

    context "when method is get" do
      it "returns an http response" do
        path = '/oauth/authorize'
        params = {:client_id => '001337', :client_secret => 'abcxyz'}
        method = :get

        normalized_path = '/oauth/authorize?client_id=001337&client_secret=abcxyz'

        stub_request(:get, "https://plangrade.com/oauth/authorize?client_id=001337&client_secret=abcxyz").with(
          :headers => {
            'Accept'=>'application/json',
            'User-Agent'=>"Plangrade Ruby Gem #{Plangrade::Ruby::VERSION}"
        })
        response = subject.send(:request, method, path, params)

        expect(response.code).to eq 200
      end
    end

    context "when method is delete" do
      it "returns an http response" do
        path = '/users/1'

        stub_request(:delete, "https://plangrade.com/users/1").with(
          :headers => {
            'Accept'=>'application/json',
            'User-Agent'=>"Plangrade Ruby Gem #{Plangrade::Ruby::VERSION}"
          }).to_return(:status => 200, :body => "", :headers => {})

        response = subject.send(:request, :delete, path)
        expect(response.code).to eq 200
      end
    end

    context "when method is post" do
      it "returns an http response" do
        path = '/participants'
        params = {:company_id =>1, :first_name => 'Christopher'}
        query  = Addressable::URI.form_encode(params)
        headers = {'Content-Type' => 'application/x-www-form-urlencoded' }.merge(subject.default_headers)

        stub_request(:post, "https://plangrade.com/participants").with(
          :body => {
            "company_id"=>"1",
            "first_name"=>"Christopher"
            },
          :headers => {
            'Accept'=>'application/json',
            'Content-Type'=>'application/x-www-form-urlencoded',
            'User-Agent'=>"Plangrade Ruby Gem #{Plangrade::Ruby::VERSION}"
        }).to_return(:status => 200, :body => "", :headers => {})

        response =subject.send(:request, :post, path, params)
        expect(response.code).to eq 200
      end
    end

    context "when method is put" do
      it "returns an http response" do
        path = '/users/1'
        params = {:name => 'christopher reynoso'}
        query  = Addressable::URI.form_encode(params)
        headers = {'Content-Type' => 'application/x-www-form-urlencoded' }.merge(subject.default_headers)

        stub_request(:put, "https://plangrade.com/users/1").with(
          :body => {
            "name" => "christopher reynoso"
          },
          :headers => {
            'Accept'=>'application/json',
            'Content-Type'=>'application/x-www-form-urlencoded',
            'Authorization'=>'Bearer TolNOFka9Uls2DxahNi78A',
            'User-Agent'=>"Plangrade Ruby Gem #{Plangrade::Ruby::VERSION}"
          }).
         to_return(:status => 200, :body => "", :headers => {})

        response = subject.send(:request, :put, path, params)
        expect(response.code).to eq 200
      end
    end

    it "follows redirect" do
      params = {:company_id => "1", :first_name => 'Christopher'}
      stub_request(:post, "https://plangrade.com/participants").with(
        :body => params,
        :headers => {
          'Accept' =>'application/json',
          'Accept-Encoding' => 'gzip, deflate',
          'Content-Type'    => 'application/x-www-form-urlencoded',
          'User-Agent'      => "Plangrade Ruby Gem #{Plangrade::Ruby::VERSION}"
        }
      ).to_return(:status => 303, :body => "", :headers => { 'Location' => 'https://plangrade.com/users'})

      stub_request(:get, "https://plangrade.com/users").
         with(:headers => {'Accept'=>'application/json', 'Accept-Encoding'=>'gzip, deflate', 'User-Agent'=>"Plangrade Ruby Gem #{Plangrade::Ruby::VERSION}"}).
         to_return(:status => 200, :body => "", :headers => {})
      params = {:company_id => 1, :first_name => 'Christopher'}
      response = subject.send(:request, :post, '/participants', params)

      expect(response.code).to eq 200
    end

    it "respects the redirect limit " do
      subject.connection_options = { :max_redirects => 1 }

      stub_request(:get, "https://plangrade.com/participants").
         with(
          :headers => {
            'Accept' => 'application/json',
            'Accept-Encoding'=> 'gzip, deflate',
            'User-Agent'     => "Plangrade Ruby Gem #{Plangrade::Ruby::VERSION}"
          }
        ).to_return(:status => 301, :body => "", :headers => { 'Location' => 'https://plangrade.com/users'})

       stub_request(:get, "https://plangrade.com/users").
         with(
          :headers => {
            'Accept' => 'application/json',
            'Accept-Encoding'=> 'gzip, deflate',
            'User-Agent'     => "Plangrade Ruby Gem #{Plangrade::Ruby::VERSION}"
          }
        ).to_return(:status => 301, :body => "", :headers => { 'Location' => 'https://plangrade.com/companies'})

      expect { subject.send(:request, :get, '/participants') }.to raise_error(RestClient::MaxRedirectsReached)
    end
  end

  context 'http request' do
    describe "#post" do
      it "makes an http POST request" do
        stub_post('/users').with(
          :headers => {
            'Accept'=>'application/json',
            'Authorization'=>'Bearer TolNOFka9Uls2DxahNi78A',
            'Content-Type'=>'application/x-www-form-urlencoded',
            'User-Agent'=>"Plangrade Ruby Gem #{Plangrade::Ruby::VERSION}"
          },
          :body => { :name => 'christopher reynoso' }).to_return({
            :status => 200, :body => "", :headers => {}
          })
        subject.post('/users', { :name => 'christopher reynoso'})
      end
    end

    describe "#put" do
      it "makes an http PUT request" do
        stub_put('/users/1').with(
          :headers => {
            'Accept'=>'application/json',
            'Authorization'=>'Bearer TolNOFka9Uls2DxahNi78A',
            'Content-Type'=>'application/x-www-form-urlencoded',
            'User-Agent'=> "Plangrade Ruby Gem #{Plangrade::Ruby::VERSION}"
          },
          :body => { :name => 'topher reynoso' }).to_return(
            :status => 200, :body => "", :headers => {})

        subject.put('/users/1', { :name => 'topher reynoso'})
      end
    end

    describe "#get" do
      it "makes an http GET request" do
        stub_get('/users/1').with(:headers => { 'Authorization' => 'Bearer TolNOFka9Uls2DxahNi78A' })
        subject.get('/users/1')
      end
    end

    describe "#delete" do
      it "makes an http DELETE request" do
        stub_delete('/users/1').with(:headers => { 'Authorization' => 'Bearer TolNOFka9Uls2DxahNi78A' })
        subject.delete('/users/1')
      end
    end

    describe 'with invalid token' do
      it 'raises exception' do
        stub_get('/users/1').with(:headers => { 'Authorization' => 'Bearer TolNOFka9Uls2DxahNi78A' }).to_return(
          :body => '{ "response": { "message": "Token not found.", "code": 16, "stat": "fail" } }',
          :status => 401)

        expect(subject.get('/users/1')).to eq('{ "response": { "message": "Token not found.", "code": 16, "stat": "fail" } }')
      end
    end

    describe 'with too many requests' do
      it 'raises exception' do
        stub_get('/users/1').with(:headers => { 'Authorization' => 'Bearer TolNOFka9Uls2DxahNi78A' }).to_return(
          :body => '{ "response": { "message": "Rate limited due to excessive requests.", "code": 33, "stat": "fail" } }',
          :status => 429
        )
        expect(subject.get('/users/1')).to eq('{ "response": { "message": "Rate limited due to excessive requests.", "code": 33, "stat": "fail" } }')
      end
    end
  end
end